import {
    AfterViewInit,
    Component,
    ElementRef,
    OnInit,
    ViewChild
} from '@angular/core';

import { ActivatedRoute } from '@angular/router';

import {
    MatPaginator,
    MatSort,
    MatTableDataSource
} from '@angular/material';

import { DatePipe } from '@angular/common';

import {
    debounceTime,
    distinctUntilChanged,
    startWith,
    tap,
    delay
} from 'rxjs/operators';

import { merge } from 'rxjs/observable/merge';
import { fromEvent } from 'rxjs/observable/fromEvent';

import { saveAs } from 'file-saver';

import { MerchandiseService } from '../services/merchandise.service';
import { MerchandiseDataSource } from '../services/merchandise.datasource';


@Component({
    selector: 'merchandise',
    templateUrl: './merchandise.component.html',
    styleUrls: [ './merchandise.component.css' ]
})
export class MerchandiseComponent implements OnInit, AfterViewInit {
    dataSource: MerchandiseDataSource;

    displayedColumns = [
        'name'
    ];

    default_variants = [];
    product_urls = [];
    front_images = [];
    back_images = [];

    @ViewChild(MatPaginator) paginator: MatPaginator;
    @ViewChild(MatSort) sort: MatSort;
    @ViewChild('input') input: ElementRef;

    constructor(private route: ActivatedRoute,
                private merchandiseService: MerchandiseService) { }

    ngOnInit() {
        this.dataSource = new MerchandiseDataSource(this.merchandiseService);
        this.dataSource.loadProducts('', 'id', 'asc', 0, 5);

        this.dataSource.connect(null)
            .subscribe(data => {
                this.default_variants = [];
                this.front_images = [];
                this.product_urls = [];
                this.back_images = [];

                for (let product of data) {
                    this.default_variants[product.id] = null;
                    this.product_urls[product.id] = product.url;
                    this.front_images[product.id] = product.image_url;
                    this.back_images[product.id] = null;

                    for (let variant of product.variants) {
                        if (product.url.indexOf(variant.id.toString()) != -1) {
                            this.default_variants[product.id] = variant.id;
                            this.front_images[product.id] = variant.front_url;
                            this.back_images[product.id] = variant.back_url;
                            break;
                        }
                    }
                }
            });
    }

    ngAfterViewInit() {
        this.sort.sortChange.subscribe(() => this.paginator.pageIndex = 0);

        fromEvent(this.input.nativeElement, 'keyup')
            .pipe(
                debounceTime(150),
                distinctUntilChanged(),
                tap(() => {
                    this.paginator.pageIndex = 0;
                    this.loadProductsPage();
                })
            )
            .subscribe();

        merge(this.sort.sortChange, this.paginator.page)
            .pipe(
                tap(() => this.loadProductsPage())
            )
            .subscribe();
    }

    loadProductsPage() {
        this.dataSource.loadProducts(
            this.input.nativeElement.value,
            this.sort.active,
            this.sort.direction,
            this.paginator.pageIndex,
            this.paginator.pageSize);
    }

    switch_variant(product, variant) {
        this.product_urls[product.id] = product.url;

        let default_variant = this.default_variants[product.id];

        if (default_variant) {
            this.product_urls[product.id] = product.url.replace(
                default_variant.toString(), variant.id.toString());
        }

        this.front_images[product.id] = variant.front_url;
        this.back_images[product.id] = variant.back_url;
    }
}
